@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_xml_parse.c,v 1.29 2003/02/26 10:12:24 hvr Exp $
+    $Id: vcd_xml_parse.c,v 1.32 2003/11/10 11:59:13 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -23,6 +23,7 @@
 #endif
 
 #include "vcd_xml_parse.h"
+#include "vcd_xml_common.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -30,10 +31,13 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
-#include <libvcd/vcd_util.h>
-#include <libvcd/vcd_logging.h>
+#include <libvcd/logging.h>
 
-static const char _rcsid[] = "$Id: vcd_xml_parse.c,v 1.29 2003/02/26 10:12:24 hvr Exp $";
+/* Private headers */
+#include "util.h"
+#include "vcd_xml_parse.h"
+
+static const char _rcsid[] = "$Id: vcd_xml_parse.c,v 1.32 2003/11/10 11:59:13 rocky Exp $";
 
 /*
  * shortcut templates...
@@ -207,7 +211,9 @@ _parse_mpeg_segment (struct vcdxml_t *obj, xmlDocPtr doc, xmlNodePtr node, xmlNs
   _vcd_list_append (obj->segment_list, segment);
 
   GET_PROP_STR (segment->id, "id", doc, node, ns);
+  segment->src = vcd_xml_utf8_to_filename ((unsigned char *)segment->src); // memleak
   GET_PROP_STR (segment->src, "src", doc, node, ns);
+  segment->src = vcd_xml_utf8_to_filename ((unsigned char *)segment->src); // memleak
 
   segment->autopause_list = _vcd_list_new ();
 
@@ -518,7 +524,9 @@ _parse_mpeg_sequence (struct vcdxml_t *obj, xmlDocPtr doc, xmlNodePtr node, xmlN
   _vcd_list_append (obj->sequence_list, sequence);
 
   GET_PROP_STR (sequence->id, "id", doc, node, ns);
+  sequence->src = vcd_xml_utf8_to_filename ((unsigned char *)sequence->src); // memleak
   GET_PROP_STR (sequence->src, "src", doc, node, ns);
+  sequence->src = vcd_xml_utf8_to_filename ((unsigned char *)sequence->src); // memleak
 
   sequence->entry_point_list = _vcd_list_new ();
   sequence->autopause_list = _vcd_list_new ();
@@ -597,7 +605,9 @@ _parse_file (struct vcdxml_t *obj, const char path[], xmlDocPtr doc, xmlNodePtr
   vcd_assert (path != NULL);
 
   GET_PROP_STR (_src, "src", doc, node, ns);
+  _src = vcd_xml_utf8_to_filename ((unsigned char *) _src); // memleak
   vcd_assert (_src != NULL);
+  _src = vcd_xml_utf8_to_filename ((unsigned char *) _src); // memleak
 
   GET_PROP_STR (_format, "format", doc, node, ns);
 
